<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>{% block title %}{% endblock %}</title>
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.6.0/jquery.min.js"></script>
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.2.0-beta1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-0evHe/X+R7YkIZDRvuzKMRqM+OrBnVFBL6DOitfPri4tjfHxaWutUpFmBp4vmVor" crossorigin="anonymous">
    <link href="{{ url_for('static', filename='css/base.css') }}" rel="stylesheet" type="text/css">
    {% block head %}
    {% endblock %}
</head>
<body>
    <div id="navbar">
        <nav class="navbar navbar-dark bg-primary">
            <div class="container-fluid">
              <a class="navbar-brand" href="/">SQL Express</a>
            </div>
        </nav>
    </div>
    <div id="content" class="content">
        {% block content %}
        {% endblock %}
    </div>
    <div id="footer">
        <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.2.0-beta1/dist/js/bootstrap.bundle.min.js" integrity="sha384-pprn3073KE6tl6bjs2QrFaJGz5/SUsLqktiwsUTF55Jfv3qYSDhgCecCxMW52nD2" crossorigin="anonymous"></script>
        {% block footer %}
        {% endblock %}
    </div>
</body>
</html>