h1 {
    margin-top: 30px;
    margin-bottom: 30px;
}

.content {
    margin: 30px;
}

.project-nav {
    margin-top: 30px;
}

.project-actions {
    margin-top: 30px;
}

.project-content {
    margin-top: 30px;
}