{% extends "project.html" %}

{% block title %}Project - Output{% endblock %}

{% block project_content %}
    <div class="card border-primary">
      <div class="card-body">
        <pre>{{ project.output }}</pre>
      </div>
    </div>
{% endblock %}