{% extends "base.html" %}

{% block title %}Project{% endblock %}

{% block content %}

    <h1>File: {{ project.filename }}</h1>

    <div id="project-nav" class="project-nav">
        <ul class="nav nav-pills project-pills">
          <li class="nav-item">
            <a class="nav-link" href="{{ url_for('project_input', filename=project.filename) }}">
                Input
            </a>
          </li>
          <li class="nav-item">
            <a class="nav-link" href="{{ url_for('project_output', filename=project.filename) }}">
                Output
            </a>
          </li>
          <li class="nav-item">
            <a class="nav-link" href="{{ url_for('project_graph', filename=project.filename) }}">
                Graph
            </a>
          </li>
          <li class="nav-item">
            <a class="nav-link" href="{{ url_for('project_tables', filename=project.filename) }}">
                Tables
            </a>
          </li>
        </ul>
    </div>

    <div id="project-actions" class="project-actions">
        {% block project_actions %}
        {% endblock %}
    </div>

    <div id="project-content" class="project-content">
        {% block project_content %}
        {% endblock %}
    </div>

{% endblock %}

{% block footer %}
    <script src="{{ url_for('static', filename='js/project.js') }}"></script>
{% endblock %}
